#include "stdafx.h"
#include "ContextMenu.h"

STDMETHODIMP CContextMenu::Initialize (
	LPCITEMIDLIST /*pidlFolder*/,
	LPDATAOBJECT  pDO,
	HKEY          /*hkeyProgID*/ )
{
	if (!m_pIImgCtx)
		return E_FAIL;
	
	if (!pDO)
		return E_INVALIDARG;

	//     
	STGMEDIUM med;
	FORMATETC fe = { CF_HDROP, NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
	HRESULT hr = pDO->GetData (&fe, &med);
	if (SUCCEEDED (hr)) {
		hr = E_FAIL;

		//       
		if (DragQueryFile ((HDROP) med.hGlobal, 0xFFFFFFFF, 0, 0) == 1) {
			UINT res = DragQueryFile ((HDROP) med.hGlobal, 0,
				m_szFile.GetBuffer (MAX_PATH), MAX_PATH);
			m_szFile.ReleaseBuffer ();

			//     10 
			WIN32_FILE_ATTRIBUTE_DATA wfa;
			if (res && GetFileAttributesEx (m_szFile, GetFileExInfoStandard, &wfa) &&
				!(wfa.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
				!wfa.nFileSizeHigh && (wfa.nFileSizeLow < 1024 * 1024 * 10)) {

				//   
				hr = m_pIImgCtx->Load (m_szFile, 0);
				if (SUCCEEDED (hr)) {

					//    
					DWORD dwState = 0;					
					for (int i = 0; i < 10; ++i) {
						hr = m_pIImgCtx->GetStateInfo (&dwState, &m_ImgSize, true);
						if (SUCCEEDED (hr)) {

							//    ?
							if (IMGLOAD_NOTLOADED & dwState || IMGLOAD_LOADING & dwState) {															
								if (m_ImgSize.cx && m_ImgSize.cy) {									
									
									//        
									//  
									break;
								}
							} else
								//   ?
								if (IMGLOAD_COMPLETE & dwState) {
									
									//     
									hr = (m_ImgSize.cx && m_ImgSize.cy) ? S_OK : E_FAIL;
									break;
								}
						} else
							//  
							break;

						hr = E_FAIL;
						Sleep (100);
					}
				}
			}
		}
		ReleaseStgMedium (&med);
	}
	if (FAILED (hr)) {
		m_pIImgCtx->Release ();
		m_pIImgCtx = NULL;
	}
	return hr;
}

STDMETHODIMP CContextMenu::QueryContextMenu (
	HMENU hmenu, UINT uIndex, UINT uidCmdFirst,
	UINT /*uidCmdLast*/, UINT uFlags )
{
	// If the flags include CMF_DEFAULTONLY then we shouldn't do anything.
	if (uFlags & CMF_DEFAULTONLY)
		return MAKE_HRESULT (SEVERITY_SUCCESS, FACILITY_NULL, 0);

	MENUITEMINFO mii;
	mii.cbSize = sizeof (MENUITEMINFO);
	mii.fMask  = MIIM_ID | MIIM_TYPE;
	mii.fType  = MFT_OWNERDRAW;
	mii.wID    = uidCmdFirst;
	if (!InsertMenuItem (hmenu, uIndex, TRUE, &mii))
		return MAKE_HRESULT (SEVERITY_SUCCESS, FACILITY_NULL, 0);

	// Store the menu item's ID so we can check against it later when
	// WM_MEASUREITEM/WM_DRAWITEM are sent. 
	m_uOurItemID = uidCmdFirst;

	// Tell the shell we added 1 top-level menu item.
	return MAKE_HRESULT (SEVERITY_SUCCESS, FACILITY_NULL, 1);
}

STDMETHODIMP CContextMenu::GetCommandString (
	UINT uCmd, UINT uFlags, UINT* /*puReserved*/,
	LPSTR pszName, UINT cchMax )
{
	USES_CONVERSION;

	CString HelpString;
	HelpString.LoadString (IDS_TIP);

	// Check idCmd, it must be 0 since we have only one menu item.
	if (0 != uCmd)
		return E_INVALIDARG;

	// If Explorer is asking for a help string, copy our string into the
	// supplied buffer.
	if (uFlags & GCS_HELPTEXT)
		if (uFlags & GCS_UNICODE)
			// We need to cast pszName to a Unicode string, and then use the
			// Unicode string copy API.
			lstrcpynW ((LPWSTR) pszName, T2CW (HelpString), cchMax);
		else
			// Use the ANSI string copy API to return the help string.
			lstrcpynA (pszName, T2CA (HelpString), cchMax);

	return S_OK;
}

STDMETHODIMP CContextMenu::InvokeCommand ( LPCMINVOKECOMMANDINFO pInfo )
{
	// If lpVerb really points to a string, ignore this function call and bail out.
	if (0 != HIWORD (pInfo->lpVerb))
		return E_INVALIDARG;

	// The command ID must be 0 since we only have one menu item.
	if (0 != LOWORD (pInfo->lpVerb))
		return E_INVALIDARG;

	// Open the bitmap in the default paint program.
	ShellExecute (pInfo->hwnd, _T("open"), m_szFile, NULL, NULL, SW_SHOWNORMAL);

	return S_OK;
}

STDMETHODIMP CContextMenu::HandleMenuMsg ( UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	// res is a dummy LRESULT variable.  It's not used (IContextMenu2::HandleMenuMsg()
	// doesn't provide a way to return values), but it's here so that the code 
	// in MenuMessageHandler() can be the same regardless of which interface it
	// gets called through (IContextMenu2 or 3).

	LRESULT res;
	return MenuMessageHandler (uMsg, wParam, lParam, &res);
}

STDMETHODIMP CContextMenu::HandleMenuMsg2 ( UINT uMsg, WPARAM wParam, LPARAM lParam,
											 LRESULT* pResult )
{
	// For messages that have no return value, pResult is NULL.  This is
	// _very_ mean on the part of MS since it forces us to check whether
	// pResult is valid before using it.  You'd think that a pointer to
	// a "return value" would always be valid, but alas no.
	// If it is NULL, I create a dummy LRESULT variable, so the code in
	// MenuMessageHandler() will always have a valid pResult pointer.

	LRESULT res;
	return MenuMessageHandler ( uMsg, wParam, lParam, pResult ? pResult : &res);
}

STDMETHODIMP CContextMenu::MenuMessageHandler (UINT uMsg, WPARAM /*wParam*/,
												 LPARAM lParam, LRESULT* pResult)
{
	switch (uMsg) {
		case WM_MEASUREITEM:
			return OnMeasureItem ((MEASUREITEMSTRUCT*) lParam, pResult);

		case WM_DRAWITEM:
			return OnDrawItem ((DRAWITEMSTRUCT*) lParam, pResult);
	}
	return S_OK;
}

STDMETHODIMP CContextMenu::OnMeasureItem ( MEASUREITEMSTRUCT* pmis, LRESULT* pResult )
{
	// Check that we're getting called for our own menu item.
	if ( m_uOurItemID != pmis->itemID )
		return S_OK;

	//   
	if (m_ImgSize.cx > m_ImgSize.cy) {
		if (m_ImgSize.cx < m_ThumbSize.cx) {
			m_PicSize.cx = m_ImgSize.cx;
			m_PicSize.cy = m_ImgSize.cy;
		} else {
			m_PicSize.cx = m_ThumbSize.cx;
			m_PicSize.cy = (m_ThumbSize.cy * m_ImgSize.cy) /
				(m_ImgSize.cx);
		}
	} else {
		if (m_ImgSize.cy < m_ThumbSize.cy) {
			m_PicSize.cx = m_ImgSize.cx;
			m_PicSize.cy = m_ImgSize.cy;
		} else {
			m_PicSize.cx = (m_ThumbSize.cx * m_ImgSize.cx) /
				(m_ImgSize.cy);
			m_PicSize.cy = m_ThumbSize.cy;
		}
	}
	pmis->itemWidth = m_ItemSize.cx = m_ThumbSize.cx + 4;
	pmis->itemHeight = m_ItemSize.cy = m_ThumbSize.cy + 24;

	*pResult = TRUE;
	return S_OK;
}

STDMETHODIMP CContextMenu::OnDrawItem (DRAWITEMSTRUCT* pdis, LRESULT* pResult)
{
	// Check that we're getting called for our own menu item.
	if (m_uOurItemID != pdis->itemID)
		return S_OK;

	//      
	DWORD dwState = 0;
	BOOL downloading =
		SUCCEEDED (m_pIImgCtx->GetStateInfo (&dwState, NULL, true)) &&
		(IMGLOAD_NOTLOADED & dwState || IMGLOAD_LOADING & dwState);

	RECT rcItem = {
		pdis->rcItem.left + 1, pdis->rcItem.top + 1,
		pdis->rcItem.right - 1, pdis->rcItem.bottom - 21
	};
	RECT rcText = {
		rcItem.left, rcItem.bottom + 2, rcItem.right, pdis->rcItem.bottom - 2
	};
	RECT rcDraw;
	rcDraw.left = (rcItem.right + rcItem.left - m_PicSize.cx) / 2;
	rcDraw.top = (rcItem.top + rcItem.bottom - m_PicSize.cy) / 2;
	rcDraw.right = rcDraw.left + m_PicSize.cx;
	rcDraw.bottom = rcDraw.top + m_PicSize.cy;

	CString str;
	str.Format (_T("%dx%d%s"), m_ImgSize.cx, m_ImgSize.cy,
		(downloading ? _T(" Loading...") : _T("")));

	// 
	HBRUSH FillBrush = CreateSolidBrush (GetSysColor (
		(pdis->itemState & ODS_SELECTED) ? COLOR_HIGHLIGHT : COLOR_3DFACE));
	FillRect (pdis->hDC, &pdis->rcItem, FillBrush);
	DrawEdge (pdis->hDC, &rcItem, EDGE_ETCHED, BF_RECT | BF_FLAT);
	m_pIImgCtx->StretchBlt (pdis->hDC, rcDraw.left, rcDraw.top,
		m_PicSize.cx, m_PicSize.cy, 0, 0, m_ImgSize.cx, m_ImgSize.cy, SRCCOPY);
	int old_mode = SetBkMode (pdis->hDC, TRANSPARENT);
	HGDIOBJ old_font = SelectObject (pdis->hDC, GetStockObject (ANSI_VAR_FONT));
	FillRect (pdis->hDC, &rcText, FillBrush);
	DrawEdge (pdis->hDC, &rcText, EDGE_ETCHED, BF_RECT | BF_FLAT);
	DrawText (pdis->hDC, str, -1, &rcText, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
	DeleteObject (FillBrush);
	SetBkMode (pdis->hDC, old_mode);
	SelectObject (pdis->hDC, old_font);

	*pResult = TRUE;
	return S_OK;
}
